/* -*-mode:C;c-basic-offset:8-*-
 *      Copyright (C) 1993-1995 Bas Laarhoven.
 *                    
 *      changed by Claus Heine, 1996

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 $Source: /homes/cvs/zftape/tracing.c,v $
 $Author: claus $
 *
 $Revision: 1.9 $
 $Date: 1996/03/02 13:34:12 $
 $State: Exp $
 *
 *      This file contains the reading code
 *      for the QIC-117 floppy-tape driver for Linux.
 */

#ifndef NO_TRACE_AT_ALL
static char RCSid[] =
"$Id: tracing.c,v 1.9 1996/03/02 13:34:12 claus Exp $";

#include "ftape.h"
#include <linux/string.h>
/*      Global vars.
 */
/*      tracing
 *      set it to:     to get:               kernel log level:
 *      (-1)                                   KERN_ERROR
 *       0              bugs,                  KERN_WARNING
 *       1              + errors               KERN_NOTICE
 *       2              + warnings             KERN_INFO
 *       3              + information          KERN_DEBUG
 *       4              + more information     KERN_DEBUG
 *       5              + program flow         KERN_DEBUG
 *       6              + fdc/dma info         KERN_DEBUG
 *       7              + data flow            KERN_DEBUG
 *       8              + everything else      KERN_DEBUG
 *       9              + even more            KERN_DEBUG
 */
int tracing = 3;                /* sets default level */
byte trace_id = 0;
int function_nest_level = 0;
#define KERN_OFFSET 4 /* KERN_WARNING */

static inline char *
make_format_1( int level )
{
	static char format_str[40] = "< >[%03d]%*s%s (%s)\n";

	level += KERN_OFFSET;
	if ( level < KERN_OFFSET ) {
		level = KERN_OFFSET;  /* we don't want to log with less than 
				       * KERN_ERR 
				       */
	} else if ( level > 7 ) {
		level = 7;               /* 7 is KERN_DEBUG, more is not 
					  * possible 
					  */
	}
	format_str[1] = (char)(level+'0');
	return format_str;
}


static inline char *
make_format_2( int level )
{
	static char format_strs[][40] = { "< >[%03d]%*s%s (%s) - ",
					  "< >[%03d]%*s%s (%s) Error: ",
					  "< >[%03d]%*s%s (%s) Warning: "
	};
	char *format_str;

	level += KERN_OFFSET;
	if ( level < KERN_OFFSET ) {
		level = KERN_OFFSET;  /* we don't want to log with less than 
				       * KERN_ERR 
				       */
	} else if ( level > 7 ) {
		level = 7;               /* 7 is KERN_DEBUG, more is not 
					  * possible 
					  */
	}
	if ( level <= 5 ) {
		format_str = format_strs[1];
	} else if ( level == 6 ) {
		format_str = format_strs[2];
	} else {
		format_str = format_strs[0];
	}
	format_str[1] = (char)(level+'0');
	return format_str;
}

int
trace_call( int level, char* file, char* name) {
	if (tracing >= level && level <= TOP_LEVEL) {
		printk( make_format_1( level ), 
			(int) trace_id++, 
			function_nest_level,
			"+", 
			file, 
			name );
	}
	return function_nest_level++;
}

void
trace_exit( int level, char* file, char* name) {
	if (tracing >= level && level <= TOP_LEVEL) {
		printk( make_format_1( level ), 
			(int) trace_id++, 
			function_nest_level, 
			"-", 
			file, 
			name );
	}
}

void
trace_log( int level, char* file, char* name)
{
	printk( make_format_2( level ), 
		(int) trace_id++, 
		function_nest_level, 
		" ", 
		file, name );
}

#endif /* NO_TRACE_AT_ALL */



